package SPVM::Go::UV::Request;



1;

=encoding utf8

=head1 Name

SPVM::Go::UV::Request - Base Request for libuv

=head1 Description

C<Go::UV::Request> in L<SPVM> represents the base request for L<libuv|https://libuv.org/>, corresponding to the C<uv_req_t> structure.

=head1 Usage

  use Go::UV::Request;

=head1 Well Known Child Classes

=over 2

=item * L<Go::UV::Request::Write|SPVM::Go::UV::Request::Write>

=back

=head1 Copyright & License

Copyright (c) 2026 Yuki Kimoto

MIT License
