package SPVM::Go::Goroutine;



1;

=head1 Name

SPVM::Go::Goroutine - Goroutines

=head1 Description

The Go::Goroutine class in L<SPVM> has methods to manipulate goroutines.

=head1 Usage

  use Go::Goroutine;

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

